import {
  ButtonProps,
  Tooltip,
  useDisclosure,
  useToast,
  Text,
  AlertDialog,
  AlertDialogOverlay,
  AlertDialogContent,
  AlertDialogHeader,
  AlertDialogBody,
  AlertDialogFooter,
  Button,
} from '@chakra-ui/react'
import React from 'react'
import { useTranslation } from 'react-i18next'
import { Implementation, Report } from 'src/models'
import { DeleteReportMutation } from '../query-builder/delete-report-mutation'
import { capitalize } from 'lodash'
import { useQueryClient } from 'react-query'
import { GetReportsByImplementationIdQuery } from '../query-builder'

export type DeleteReportButtonProps = {
  report: Report
  implementationId: Implementation['id']
  children: React.ReactElement<ButtonProps>
}

export const DeleteReportButton = ({
  report,
  implementationId,
  children,
}: DeleteReportButtonProps) => {
  const { t } = useTranslation()
  const toast = useToast()
  const queryClient = useQueryClient()
  const [isLoading, setIsLoading] = React.useState(false)
  const { isOpen, onOpen, onClose } = useDisclosure()
  const cancelRef = React.useRef<HTMLButtonElement>(null)

  const deleteReportMutation = DeleteReportMutation.useMutation({
    onSuccess: () => {
      setIsLoading(false)
      toast({
        status: 'success',
        position: 'top',
        title: capitalize(t('reports_delete_success')),
      })
      queryClient.invalidateQueries(
        GetReportsByImplementationIdQuery.getKey({ id: implementationId })
      )
    },
    onError: () => {
      toast({
        status: 'error',
        position: 'top',
        title: `Ha ocurrido un error al eliminar el reporte.`,
      })
      onClose()
      setIsLoading(false)
    },
  })

  const handleOnDelete = () => {
    setIsLoading(true)
    deleteReportMutation.mutate({ id: report.id })
  }

  if (!React.isValidElement(children) || Array.isArray(children)) {
    throw new Error(
      'DeleteReportButton child must be a single clickable element'
    )
  }

  const btn = React.cloneElement(children, { onClick: onOpen })

  return (
    <>
      <Tooltip
        hasArrow
        shouldWrapChildren
        label={<Text>{capitalize(t('reports_delete'))}</Text>}
      >
        {btn}
      </Tooltip>
      <AlertDialog
        isOpen={isOpen}
        onClose={onClose}
        leastDestructiveRef={cancelRef}
      >
        <AlertDialogOverlay>
          <AlertDialogContent>
            <AlertDialogHeader fontSize="lg" fontWeight="bold">
              {capitalize(t('reports_delete'))}
            </AlertDialogHeader>
            <AlertDialogBody>
              <Text>
                {capitalize(t('reports_delete_report_confirmation_modal_body'))}
              </Text>
            </AlertDialogBody>
            <AlertDialogFooter>
              <Button ref={cancelRef} variant="ghost" onClick={onClose}>
                {capitalize(t('cancel'))}
              </Button>
              <Button
                variant="solid"
                onClick={handleOnDelete}
                ml="3"
                isLoading={isLoading}
              >
                {capitalize(t('delete'))}
              </Button>
            </AlertDialogFooter>
          </AlertDialogContent>
        </AlertDialogOverlay>
      </AlertDialog>
    </>
  )
}
